#!/bin/bash

set -e
# set -x

set_value_content()
{
    if [ -z $1 ] || [ -z $3 ];then
        return 1
    fi

    local VALUENAME=$1
    local VALUECONTENT=$2
    local FILEPATH=$3
    local LOGFILEPATH=$4

    grep -q "[ ]*$VALUENAME[ ]*=" "$FILEPATH"
    
    if [ $? -eq 0 ] ; then
        if [ -z $LOGFILEPATH ] ; then
            sed -i "s/\([ ]*${VALUENAME}\).*/\1=${VALUECONTENT}/" "$FILEPATH" # no loggging
        else
            sed -i "/\([ ]*${VALUENAME}\).*/{
h
s//\1=${VALUECONTENT}/g
H
x
s/\n/ >>> /
w /dev/stdout
x}" "$FILEPATH" >> "$LOGFILEPATH"
        fi
    else
        sed -i "$ a ${VALUENAME}=${VALUECONTENT}" "$FILEPATH"
        if [ ! -z $LOGFILEPATH ] ; then
            echo "+++ ${VALUENAME}=${VALUECONTENT}" | cat >> "$LOGFILEPATH"
        fi
    fi

    return 0
}

function do_config_file()
{
    local FILEPATH=$1
    shift
    local LOGFILEPATH=$1
    shift
    local -a CONFIGDEF=("$@")

    if [ ! -z $LOGFILEPATH ] ;then
        echo "" | cat >> "$LOGFILEPATH"
        echo "${FILEPATH}:" | cat >> "$LOGFILEPATH"
    fi

    cp -f "$FILEPATH"{,.bak}

    declare -i i
    i=0
    while [ $i -lt ${#CONFIGDEF[@]} ]; do
        set_value_content "${CONFIGDEF[${i}]}" "${CONFIGDEF[${i} + 1]}" "$FILEPATH" "$LOGFILEPATH"
        i=`expr $i + 2` 
    done

    return 0
}

unset_value_content()
{
    if [ -z $1 ] || [ -z $2 ];then
        return 1
    fi

    local VALUENAME=$1
    local FILEPATH=$2
    local LOGFILEPATH=$3

#    sed  -i "/\([ ]*${VALUENAME}\).*/d" "$FILEPATH" #удаляет значение
    if [ -z $LOGFILEPATH ] ; then
        sed -i "s/\([ ]*${VALUENAME}\).*/\1=/" "$FILEPATH"
    else
        sed -i "/\([ ]*${VALUENAME}\).*/{
h
s//\1=/g
H
x
s/\n/ >>> /
w /dev/stdout
x}" "$FILEPATH" >> "$LOGFILEPATH"
    fi
}

undo_config_file()
{
    local FILEPATH=$1
    shift
    local LOGFILEPATH=$1
    shift
    local -a CONFIGDEF=("$@")

    if [ ! -z $LOGFILEPATH ] ;then
        echo "" | cat >> "$LOGFILEPATH"
        echo "${FILEPATH}:" | cat >> "$LOGFILEPATH"
    fi

    cp -f "$FILEPATH"{,.bak}

    declare -i i
    i=0
    while [ $i -lt ${#CONFIGDEF[@]} ]; do
#        echo unset_value_content "${CONFIGDEF[${i}]}" "$FILEPATH" "$LOGFILEPATH"
        unset_value_content "${CONFIGDEF[${i}]}" "$FILEPATH" "$LOGFILEPATH"
        i=`expr $i + 2` 
    done

    return 0
}

config_file1="/opt/utm/transport/conf/transport.properties"
declare -a config_def1=( 
"crypto.lib.gost.keystorePassword" 
"0987654321" 
"crypto.lib.gost.keyPassword" 
"0987654321" 
"crypto.lib.pki.keystorePassword" 
"11111111" 
"crypto.lib.pki.keyPassword" 
"11111111" 
"rsa.library.path" 
"\/usr\/local\/JaCartaEgaisUtm\/libjcPKCS11-2.so" 
"gost.library.path" 
"" 
"gost.pkcs11.library.path" 
"\/usr\/local\/JaCartaEgaisUtm\/libjcPKCS11-2.so" 
"gost.pkcs11.model" 
"123" 
)

config_file2="/opt/utm/updater/conf/transport.properties"
declare -a config_def2=( 
"crypto.lib.pki.keystorePassword" 
"11111111" 
"crypto.lib.gost.keystorePassword" 
"0987654321" 
"rsa.library.path" 
"\/usr\/local\/JaCartaEgaisUtm\/libjcPKCS11-2.so" 
"transport.app.home" 
"\/opt\/utm\/transport" 
)

config_file3="/opt/utm/monitoring/conf/transport.properties"
declare -a config_def3=( 
"crypto.lib.pki.keystorePassword" 
"11111111" 
"crypto.lib.pki.keyPassword" 
"11111111" 
"rsa.library.path" 
"\/usr\/local\/JaCartaEgaisUtm\/libjcPKCS11-2.so" 
)

config_file4="/opt/utm/agent/conf/agent.properties"
declare -a config_def4=( 
"crypto.lib.pki.keystorePassword" 
"11111111" 
"crypto.lib.gost.keystorePassword" 
"0987654321" 
"rsa.library.path" 
"\/usr\/local\/JaCartaEgaisUtm\/libjcPKCS11-2.so" 
"gost.pkcs11.library.path" 
"\/usr\/local\/JaCartaEgaisUtm\/libjcPKCS11-2.so" 
)

install_egais_utm()
{
    do_config_file $config_file1 "$1" "${config_def1[@]}" 
    do_config_file $config_file2 "$1" "${config_def2[@]}" 
    do_config_file $config_file3 "$1" "${config_def3[@]}" 
    do_config_file $config_file4 "$1" "${config_def4[@]}" 
}

uninstall_egais_utm()
{
    undo_config_file $config_file1 "$1" "${config_def1[@]}"
    undo_config_file $config_file2 "$1" "${config_def2[@]}"
    undo_config_file $config_file3 "$1" "${config_def3[@]}"
    undo_config_file $config_file4 "$1" "${config_def4[@]}"
}

usage()
{
    echo "$0 {OPTION} [LOGFILE]"
    echo "Install JaCarta support for EGAIS UTM"
    echo "    -i, --install     install JaCarta support for EGAIS UTM"
    echo "    -u, --uninstall   uninstall JaCarta support"
    echo "    LOGFILE optional path to log file"
}

LOGFILEPATH=$2
time_stamp=$(date '+%d:%m:%Y %H:%M:%S')

if [[ $# -eq 0 ]] || [[ $# -gt 2 ]] ; then
    usage
elif [[ "${1,,}" == *"-i" ]] || [[ "${1,,}" == *"--install" ]] ; then

    if [ ! -z $LOGFILEPATH ] ;then
        echo "JaCartaEgaisUtm: $time_stamp" | cat >> "$LOGFILEPATH"
        echo "" | cat >> "$LOGFILEPATH"
        echo "Install:" | cat >> "$LOGFILEPATH"
        echo "/usr/local/JaCartaEgaisUtm/libjcPKCS11-2.so" | cat >> "$LOGFILEPATH"
        echo "/usr/local/JaCartaEgaisUtm/libjckt2.so" | cat >> "$LOGFILEPATH"
    fi

    install_egais_utm "$LOGFILEPATH"

    if [ ! -z $LOGFILEPATH ] ;then
        echo "" | cat >> "$LOGFILEPATH"
    fi

elif [[ "${1,,}" == "-u" ]] || [[ "${1,,}" == "--uninstall" ]] ; then

    if [ ! -z $LOGFILEPATH ] ;then
        echo "JaCartaEgaisUtm: $time_stamp" | cat >> "$LOGFILEPATH"
        echo "" | cat >> "$LOGFILEPATH"
        echo "Uninstall:" | cat >> "$LOGFILEPATH"
        echo "/usr/local/JaCartaEgaisUtm/libjcPKCS11-2.so" | cat >> "$LOGFILEPATH"
        echo "/usr/local/JaCartaEgaisUtm/libjckt2.so" | cat >> "$LOGFILEPATH"
    fi

    uninstall_egais_utm "$LOGFILEPATH"

    if [ ! -z $LOGFILEPATH ] ;then
        echo "" | cat >> "$LOGFILEPATH"
    fi

fi
